#include <stdio.h>
#include <propeller.h>
#define P2_TARGET_MHZ 160
#include "sys/p2es_clock.h"
#define BAUD 230400

#define L_PIN 6
#define R_PIN 7
#define SYSTEM_CLOCK (_CLOCKFREQ)
#define SAMPLE_RATE 16000
#define DAC_MODE             (3 << 1) // DAC 16-bit dither, PWM
#define DIR_MODE             1        // Output enabled, overrides DIR
#define INIT_8BIT_DAC_VALUE  128
#define ANALOG_OUT_MODE      0x17     //  75 ohm, 2.0V DAC mode
#define SMARTPIN_DAC_MODE    (ANALOG_OUT_MODE << 16) | (INIT_8BIT_DAC_VALUE << 8) | (DIR_MODE << 6) | DAC_MODE
#define SAMPLE_PERIOD        SYSTEM_CLOCK  / SAMPLE_RATE  // CPU cycles between sample updates

void audio_init()
{
    __asm {
              wrpin   ##SMARTPIN_DAC_MODE, #L_PIN           // Config smartpin DAC mode on "left" pin
              wrpin   ##SMARTPIN_DAC_MODE, #R_PIN           // Config smartpin DAC mode on "right" pin
              wxpin   ##SAMPLE_PERIOD, #L_PIN               // Set sample period for left audio channel
              wxpin   ##SAMPLE_PERIOD, #R_PIN               // Set sample period for right audio channel
              dirh    #L_PIN                                // Enable smartpin DAC mode on left pin
              dirh    #R_PIN                                // Enable smartpin DAC mode on right pin
              setse1  #(1 << 6) | L_PIN                     // Event triggered every new sample period (when "left in pin rises")
    };
}

void audio_out(int value)
{
    __asm {
              xor     value, ##$8000
              wypin   value, #L_PIN                         // Output sample on left channel
              wypin   value, #R_PIN                         // Output sample on right channel
              waitse1
     };
}

int get_cosine(int value)
{
    __asm {
              qrotate ##$1000, value
              getqx   value
    };
    return value;
}

void main()
{
    int val = 0;
    int freq = 0x10000000;

    clkset(_SETFREQ, _CLOCKFREQ);
    _setbaud(BAUD);
    printf("audio_out: clockmode is $%x, clock frequency %u Hz\n", _SETFREQ, _CLOCKFREQ);
    audio_init();
    while (1)
    {
        audio_out(get_cosine(val));
        val += freq;
    }
}
